@@ -27,16 +27,17 @@ from flaky import flaky
 import numpy as np
 from packaging.version import parse
 import pytest
+from qtpy import PYQT6
 from qtpy.QtCore import Qt
+from qtpy.QtGui import QTextCursor
 from qtpy.QtWebEngineWidgets import WEBENGINE
 from spyder_kernels import __version__ as spyder_kernels_version
+from spyder_kernels.utils.pythonenv import is_conda_env
 import sympy
 
 # Local imports
 from spyder.config.base import running_in_ci, running_in_ci_with_conda
 from spyder.config.gui import get_color_scheme
-from spyder.config.utils import is_anaconda
-from spyder.py3compat import to_text_string
 from spyder.plugins.help.tests.test_plugin import check_text
 from spyder.plugins.ipythonconsole.tests.conftest import (
     get_conda_test_env, get_console_background_color, get_console_font_color,
@@ -159,6 +160,7 @@ def test_get_calltips(ipyconsole, qtbot, function, signature, documentation):
 
 @flaky(max_runs=3)
 @pytest.mark.auto_backend
+@pytest.mark.skipif(PYQT6, reason="Fails with PyQt6")
 def test_auto_backend(ipyconsole, qtbot):
     """Test that the automatic backend was set correctly."""
     # Wait until the window is fully up
@@ -291,7 +293,9 @@ def test_cython_client(ipyconsole, qtbot):
 @flaky(max_runs=3)
 @pytest.mark.order(1)
 @pytest.mark.environment_client
-@pytest.mark.skipif(not is_anaconda(), reason='Only works with Anaconda')
+@pytest.mark.skipif(
+    not is_conda_env(sys.prefix), reason='Only works with Anaconda'
+)
 @pytest.mark.skipif(not running_in_ci(), reason='Only works on CIs')
 @pytest.mark.skipif(not os.name == 'nt', reason='Works reliably on Windows')
 def test_environment_client(ipyconsole, qtbot):
@@ -461,7 +465,7 @@ def test_set_cwd(ipyconsole, qtbot, tmpdir):
 
     # spyder-ide/spyder#6451.
     savetemp = shell.get_cwd()
-    tempdir = to_text_string(tmpdir.mkdir("queen's"))
+    tempdir = str(tmpdir.mkdir("queen's"))
     shell.set_cwd(tempdir)
 
     # Get current directory.
@@ -483,7 +487,7 @@ def test_get_cwd(ipyconsole, qtbot, tmpdir):
 
     # spyder-ide/spyder#6451.
     savetemp = shell.get_cwd()
-    tempdir = to_text_string(tmpdir.mkdir("queen's"))
+    tempdir = str(tmpdir.mkdir("queen's"))
     assert shell.get_cwd() != tempdir
 
     # Need to escape \ on Windows.
@@ -534,7 +538,7 @@ def test_request_syspath(ipyconsole, qtbot, tmpdir):
 
     # Add a new entry to sys.path
     with qtbot.waitSignal(shell.executed):
-        tmp_dir = to_text_string(tmpdir)
+        tmp_dir = str(tmpdir)
         shell.execute("import sys; sys.path.append('%s')" % tmp_dir)
 
     # Ask for sys.path contents
@@ -548,10 +552,6 @@ def test_request_syspath(ipyconsole, qtbot, tmpdir):
     assert tmp_dir in syspath_contents
 
 
-@flaky(max_runs=10)
-@pytest.mark.skipif(
-    not sys.platform.startswith("linux"), reason="Fails on Windows and Mac"
-)
 def test_save_history_dbg(ipyconsole, qtbot):
     """Test that browsing command history is working while debugging."""
     shell = ipyconsole.get_current_shellwidget()
@@ -591,6 +591,10 @@ def test_save_history_dbg(ipyconsole, qtbot):
     qtbot.keyClick(control, Qt.Key_Up)
     assert 'aa = 10' in control.toPlainText()
 
+    # Exit debugging for proper close
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('q')
+
     # Open new widget
     ipyconsole.create_new_client()
 
@@ -609,6 +613,7 @@ def test_save_history_dbg(ipyconsole, qtbot):
 
     # Press Up arrow button and assert we get the last
     # introduced command
+    qtbot.waitUntil(lambda: shell.is_waiting_pdb_input())
     qtbot.keyClick(control, Qt.Key_Up)
     assert 'aa = 10' in control.toPlainText()
 
@@ -626,6 +631,10 @@ def test_save_history_dbg(ipyconsole, qtbot):
     qtbot.keyClick(control, Qt.Key_Up)
     assert '...:     print(1)' in control.toPlainText()
 
+    # Exit debugging for proper close
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('q')
+
 
 @flaky(max_runs=3)
 @pytest.mark.skipif(sys.platform == "darwin", reason="Hangs on Mac")
@@ -960,7 +969,7 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot, tmpdir):
     client = ipyconsole.get_current_client()
 
     fname = osp.basename(client.connection_file)
-    connection_file = to_text_string(tmpdir.join(fname))
+    connection_file = str(tmpdir.join(fname))
     shutil.copy2(client.connection_file, connection_file)
 
     ipyconsole.create_client_for_kernel(connection_file)
@@ -1312,9 +1321,9 @@ def test_pdb_ignore_lib(ipyconsole, qtbot, show_lib):
         qtbot.keyClick(control, Qt.Key_Enter)
 
     if show_lib:
-        assert 'iostream.py' in control.toPlainText()
+        assert 'write()' in control.toPlainText()
     else:
-        assert 'iostream.py' not in control.toPlainText()
+        assert 'write()' not in control.toPlainText()
     ipyconsole.set_conf('pdb_ignore_lib', True, section="debugger")
 
 
@@ -1341,7 +1350,9 @@ def test_calltip(ipyconsole, qtbot):
 @flaky(max_runs=3)
 @pytest.mark.order(1)
 @pytest.mark.test_environment_interpreter
-@pytest.mark.skipif(not is_anaconda(), reason='Only works with Anaconda')
+@pytest.mark.skipif(
+    not is_conda_env(sys.prefix), reason='Only works with Anaconda'
+)
 @pytest.mark.skipif(not running_in_ci(), reason='Only works on CIs')
 @pytest.mark.skipif(not os.name == 'nt', reason='Works reliably on Windows')
 def test_conda_env_activation(ipyconsole, qtbot):
@@ -1388,7 +1399,10 @@ def test_kernel_kill(ipyconsole, qtbot, external_interpreter):
     shell = ipyconsole.get_current_shellwidget()
 
     # Wait for the restarter to start
-    qtbot.wait(3000)
+    qtbot.waitUntil(
+        lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT
+    )
+
     crash_string = 'import os, signal; os.kill(os.getpid(), signal.SIGTERM)'
 
     # Check only one comm is open
@@ -1662,7 +1676,7 @@ def test_recursive_pdb(ipyconsole, qtbot):
 
 
 @pytest.mark.skipif(
-    sys.version_info[:2] == (3, 8), reason="Fails in Python 3.8"
+    PYQT6, reason="Crashes ('QThread destroyed while running')"
 )
 def test_pdb_magics_are_recursive(ipyconsole, qtbot, tmp_path):
     """
@@ -1766,8 +1780,8 @@ def test_code_cache(ipyconsole, qtbot):
     # Same for debugging
     with qtbot.waitSignal(shell.executed):
         shell.execute('%debug print()')
+    qtbot.waitUntil(lambda: 'IPdb [' in shell._control.toPlainText())
 
-    assert 'IPdb [' in shell._control.toPlainText()
     # Send two execute requests and make sure the second one is executed
     shell.execute('time.sleep(.5)')
     shell.execute('var = 318')
@@ -1784,6 +1798,10 @@ def test_code_cache(ipyconsole, qtbot):
     # Make sure the value of var didn't change
     assert shell.get_value('var') == 318
 
+    # Exit debugging for proper close
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('q')
+
 
 @flaky(max_runs=3)
 def test_pdb_code_and_cmd_separation(ipyconsole, qtbot):
@@ -1972,13 +1990,20 @@ def test_pdb_comprehension_namespace(ipyconsole, qtbot, tmpdir):
     }
 
     shell.set_kernel_configuration("namespace_view_settings", settings)
-    namespace = shell.call_kernel(blocking=True).get_namespace_view()
+    namespace = shell.call_kernel(
+        blocking=True, timeout=SHELL_TIMEOUT
+    ).get_namespace_view()
     for key in namespace:
         assert "_spyderpdb" not in key
 
+    # Exit debugging for proper close
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('q')
+
 
 @flaky(max_runs=10)
 @pytest.mark.auto_backend
+@pytest.mark.skipif(PYQT6, reason="Fails with PyQt6")
 def test_restart_interactive_backend(ipyconsole, qtbot):
     """
     Test that we ask for a restart or not after switching to different
@@ -1996,42 +2021,49 @@ def test_restart_interactive_backend(ipyconsole, qtbot):
 
     # Switch to the tk backend
     ipyconsole.set_conf('pylab/backend', 'tk')
+    qtbot.wait(500)
     assert bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
-    assert shell.get_matplotlib_backend() == "qt"
+    qtbot.waitUntil(lambda: shell.get_matplotlib_backend() == "qt")
     assert matplotlib_status.value == "Qt"
 
     # Switch to the inline backend
     os.environ.pop('BACKEND_REQUIRE_RESTART')
     ipyconsole.set_conf('pylab/backend', 'inline')
+    qtbot.wait(500)
     assert not bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
     qtbot.waitUntil(lambda: shell.get_matplotlib_backend() == "inline")
     assert matplotlib_status.value == "Inline"
 
     # Switch to the auto backend
     ipyconsole.set_conf('pylab/backend', 'auto')
+    qtbot.wait(500)
     assert not bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
     qtbot.waitUntil(lambda: shell.get_matplotlib_backend() == "qt")
     assert matplotlib_status.value == "Qt"
 
     # Switch to the qt backend
     ipyconsole.set_conf('pylab/backend', 'qt')
+    qtbot.wait(500)
     assert not bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
     assert matplotlib_status.value == "Qt"
 
     # Switch to the tk backend again
     ipyconsole.set_conf('pylab/backend', 'tk')
+    qtbot.wait(500)
     assert bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
 
-    # Check we no spurious messages are shown before the restart below
+    # Check no spurious messages are shown before the restart below
     assert empty_console_text == shell._control.toPlainText()
 
     # Restart kernel to check if the new interactive backend is set
-    ipyconsole.restart_kernel()
-    qtbot.waitUntil(lambda: shell.spyder_kernel_ready, timeout=SHELL_TIMEOUT)
-    qtbot.wait(SHELL_TIMEOUT)
+    # Don't check `matplotlib_status.value` since it is set to "inline" when
+    # running tests on CI.
+    # See `spyder.plugins.ipythonconsole.widgets.status.MatplotlibStatus.on_kernel_start`
+    with qtbot.waitSignal(shell.sig_kernel_is_ready, timeout=SHELL_TIMEOUT):
+        ipyconsole.restart_kernel()
+    assert shell.spyder_kernel_ready
     qtbot.waitUntil(lambda: shell.get_matplotlib_backend() == "tk")
     assert shell.get_mpl_interactive_backend() == "tk"
-    assert matplotlib_status.value == "Tk"
 
 
 def test_mpl_conf(ipyconsole, qtbot):
@@ -2362,7 +2394,9 @@ def test_run_script(ipyconsole, qtbot, tmp_path):
         assert sw.get_value(variable_name) == 1
 
 
-@pytest.mark.skipif(not is_anaconda(), reason="Only works with Anaconda")
+@pytest.mark.skipif(
+    not is_conda_env(sys.prefix), reason="Only works with Anaconda"
+)
 def test_show_spyder_kernels_error_on_restart(ipyconsole, qtbot):
     """Test that we show Spyder-kernels error message on restarts."""
     # Wait until the window is fully up
@@ -2423,6 +2457,37 @@ def test_restore_tmpdir(ipyconsole, qtbot, tmp_path):
     assert shell.get_value('tmpdir') == system_tmpdir
 
 
+def test_floats_selected_on_double_click(ipyconsole, qtbot):
+    """
+    Check that doing a double click on floats selects the entire number.
+
+    This is a regression test for issue spyder-ide/spyder#22207
+    """
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(
+        lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT
+    )
+
+    # Write a floating point number
+    control = shell._control
+    float_number = "10.999e5"
+    qtbot.keyClicks(control, float_number)
+
+    # Move the cursor to the left a bit
+    for __ in range(3):
+        control.moveCursor(QTextCursor.Left, mode=QTextCursor.MoveAnchor)
+
+    # Perform a double click and check the entire number was selected
+    qtbot.mouseDClick(control.viewport(), Qt.LeftButton)
+    assert control.get_selected_text() == float_number
+
+    # Move cursor at the beginning and check the number is also selected
+    qtbot.keyClick(control, Qt.Key_Home)
+    qtbot.mouseDClick(control.viewport(), Qt.LeftButton)
+    assert control.get_selected_text() == float_number
+
+
 def test_filter_frames_in_tracebacks(ipyconsole, qtbot, tmp_path):
     """Check that we filter some unnecessary frames in tracebacks."""
     # Wait until the window is fully up
@@ -2516,5 +2581,97 @@ def test_case_sensitive_wdir(ipyconsole, qtbot, tmp_path):
     assert "sensitivecasea" not in control.toPlainText()
 
 
+@flaky(max_runs=10)
+@pytest.mark.skipif(not sys.platform == "darwin", reason="Only works on Mac")
+def test_time_elapsed(ipyconsole, qtbot, tmp_path):
+    """Test that the IPython console elapsed timer is set correctly."""
+    # Create a new IPython console client
+    ipyconsole.create_new_client()
+
+    # Get the main console widget and the current shell
+    main_widget = ipyconsole.get_widget()
+    shell = ipyconsole.get_current_shellwidget()
+
+    # Enable the elapsed time display for the current client
+    main_widget.set_show_elapsed_time_current_client(True)
+
+    # Check that the elapsed time starts at 00:00:00
+    assert '00:00:00' in main_widget.time_label.text()
+
+    # Wait until the kernel is ready and wait 5 seconds to let the timer advance
+    qtbot.waitUntil(
+        lambda: shell.spyder_kernel_ready and shell._prompt_html is not None,
+        timeout=SHELL_TIMEOUT
+    )
+    qtbot.wait(5000)
+
+    # Check that elapsed time has advanced (at least to 00:00:04)
+    assert (
+        '00:00:04' in main_widget.time_label.text()
+        or '00:00:05' in main_widget.time_label.text()
+    )
+
+    # Restart the kernel
+    main_widget.restart_action.trigger()
+    qtbot.waitUntil(
+        lambda: shell.spyder_kernel_ready and shell._prompt_html is not None,
+        timeout=SHELL_TIMEOUT
+    )
+    qtbot.wait(1000)
+
+    # Verify the timer is reset to 00:00:00 after kernel restart
+    assert '00:00:00' in main_widget.time_label.text()
+
+    # Disable the elapsed time display for the current client
+    main_widget.set_show_elapsed_time_current_client(False)
+
+    # Verify that the elapsed time label is now hidden (empty text)
+    assert '' == main_widget.time_label.text()
+
+    # The test fails after this point on CIs for Mac but works locally
+    if running_in_ci() and sys.platform == "darwin":
+        return
+
+    # Enable the elapsed time display for all clients
+    ipyconsole.set_conf("show_elapsed_time", True)
+
+    # Create a new client to test that the global config is applied
+    ipyconsole.create_new_client()
+    qtbot.waitUntil(
+        lambda: shell.spyder_kernel_ready and shell._prompt_html is not None,
+        timeout=SHELL_TIMEOUT
+    )
+
+    # Check the timer starts at 00:00:00 for the new client
+    assert '00:00:00' in main_widget.time_label.text()
+
+    # Wait 5 seconds again
+    qtbot.wait(5000)
+
+    # Verify the timer has advanced
+    if os.name == "nt":
+        assert (
+            "00:00:04" in main_widget.time_label.text()
+            or "00:00:05" in main_widget.time_label.text()
+        )
+    else:
+        # This second timer is very flaky on Mac, so we simply test that it's
+        # not stuck at 0
+        '00:00:00' not in main_widget.time_label.text()
+
+    # Disable the global config for elapsed time
+    ipyconsole.set_conf("show_elapsed_time", False)
+
+    # Create another new client to verify elapsed time is disabled
+    ipyconsole.create_new_client()
+    qtbot.waitUntil(
+        lambda: shell.spyder_kernel_ready and shell._prompt_html is not None,
+        timeout=SHELL_TIMEOUT
+    )
+
+    # Check that the elapsed time is not shown
+    assert '' == main_widget.time_label.text()
+
+
 if __name__ == "__main__":
     pytest.main()
