@@ -55,6 +55,11 @@ def data(cm, i, j):
     return cm.data(cm.index(i, j))
 
 
+def data_col(cm, j):
+    n_rows = cm.rowCount()
+    return [cm.data(cm.index(i, j)) for i in range(n_rows)]
+
+
 def data_table(cm, n_rows, n_cols):
     return [[data(cm, i, j) for i in range(n_rows)] for j in range(n_cols)]
 
@@ -91,15 +96,15 @@ def test_rename_variable(qtbot):
                  'e': 5}
     editor = CollectionsEditorTableView(None, variables.copy())
     qtbot.addWidget(editor)
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
 
     editor.rename_item(new_name='b2')
-    assert editor.model.rowCount() == 5
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'b2'
-    assert data(editor.model, 2, 0) == 'c'
-    assert data(editor.model, 3, 0) == 'd'
-    assert data(editor.model, 4, 0) == 'e'
+    assert editor.model().rowCount() == 5
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'c'
+    assert data(editor.model(), 2, 0) == 'd'
+    assert data(editor.model(), 3, 0) == 'e'
+    assert data(editor.model(), 4, 0) == 'b2'
 
     # Reset variables and try renaming one again
     new_variables = {'a': 1,
@@ -110,15 +115,15 @@ def test_rename_variable(qtbot):
                      'e': 5}
     editor.set_data(new_variables.copy())
     editor.adjust_columns()
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
     editor.rename_item(new_name='b3')
-    assert editor.model.rowCount() == 6
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'b2'
-    assert data(editor.model, 2, 0) == 'b3'
-    assert data(editor.model, 3, 0) == 'c'
-    assert data(editor.model, 4, 0) == 'd'
-    assert data(editor.model, 5, 0) == 'e'
+    assert editor.model().rowCount() == 6
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'b2'
+    assert data(editor.model(), 2, 0) == 'c'
+    assert data(editor.model(), 3, 0) == 'd'
+    assert data(editor.model(), 4, 0) == 'e'
+    assert data(editor.model(), 5, 0) == 'b3'
 
 
 def test_remove_variable(qtbot):
@@ -130,25 +135,25 @@ def test_remove_variable(qtbot):
                  'e': 5}
     editor = CollectionsEditorTableView(None, variables.copy())
     qtbot.addWidget(editor)
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
 
     editor.remove_item(force=True)
-    assert editor.model.rowCount() == 4
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'c'
-    assert data(editor.model, 2, 0) == 'd'
-    assert data(editor.model, 3, 0) == 'e'
+    assert editor.model().rowCount() == 4
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'c'
+    assert data(editor.model(), 2, 0) == 'd'
+    assert data(editor.model(), 3, 0) == 'e'
 
     # Reset variables and try removing one again
     editor.set_data(variables.copy())
     editor.adjust_columns()
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
     editor.remove_item(force=True)
-    assert editor.model.rowCount() == 4
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'c'
-    assert data(editor.model, 2, 0) == 'd'
-    assert data(editor.model, 3, 0) == 'e'
+    assert editor.model().rowCount() == 4
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'c'
+    assert data(editor.model(), 2, 0) == 'd'
+    assert data(editor.model(), 3, 0) == 'e'
 
 
 def test_remove_remote_variable(qtbot, monkeypatch):
@@ -180,7 +185,7 @@ def test_remove_remote_variable(qtbot, monkeypatch):
                        'numpy_type': 'Unknown'}}
     editor = RemoteCollectionsEditorTableView(None, variables.copy())
     qtbot.addWidget(editor)
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
 
     # Monkey patch remove variables
     def remove_values(ins, names):
@@ -212,22 +217,22 @@ def test_remove_remote_variable(qtbot, monkeypatch):
         remove_values)
 
     editor.remove_item(force=True)
-    assert editor.model.rowCount() == 4
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'c'
-    assert data(editor.model, 2, 0) == 'd'
-    assert data(editor.model, 3, 0) == 'e'
+    assert editor.model().rowCount() == 4
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'c'
+    assert data(editor.model(), 2, 0) == 'd'
+    assert data(editor.model(), 3, 0) == 'e'
 
     # Reset variables and try removing one again
     editor.set_data(variables.copy())
     editor.adjust_columns()
-    editor.setCurrentIndex(editor.model.index(1, 0))
+    editor.setCurrentIndex(editor.model().index(1, 0))
     editor.remove_item(force=True)
-    assert editor.model.rowCount() == 4
-    assert data(editor.model, 0, 0) == 'a'
-    assert data(editor.model, 1, 0) == 'c'
-    assert data(editor.model, 2, 0) == 'd'
-    assert data(editor.model, 3, 0) == 'e'
+    assert editor.model().rowCount() == 4
+    assert data(editor.model(), 0, 0) == 'a'
+    assert data(editor.model(), 1, 0) == 'c'
+    assert data(editor.model(), 2, 0) == 'd'
+    assert data(editor.model(), 3, 0) == 'e'
 
 
 def test_filter_rows(qtbot):
@@ -250,23 +255,23 @@ def test_filter_rows(qtbot):
     qtbot.addWidget(editor)
 
     # Initially two rows
-    assert editor.model.rowCount() == 2
+    assert editor.model().rowCount() == 2
 
     # Match two rows by name
     editor.do_find("df")
-    assert editor.model.rowCount() == 2
+    assert editor.model().rowCount() == 2
 
     # Match two rows by type
     editor.do_find("DataFrame")
-    assert editor.model.rowCount() == 2
+    assert editor.model().rowCount() == 2
 
     # Only one match
     editor.do_find("dfb")
-    assert editor.model.rowCount() == 1
+    assert editor.model().rowCount() == 1
 
     # No match
     editor.do_find("dfbc")
-    assert editor.model.rowCount() == 0
+    assert editor.model().rowCount() == 0
 
 
 def test_remote_make_data_function():
@@ -283,7 +288,7 @@ def test_remote_make_data_function():
     editor = RemoteCollectionsEditorTableView(
         None, variables, mock_shellwidget
     )
-    index = editor.model.index(0, 0)
+    index = editor.model().index(0, 0)
     data_function = editor.delegate.make_data_function(index)
     value = data_function()
     mock_shellwidget.get_value.assert_called_once_with('a')
@@ -295,34 +300,28 @@ def test_create_dataframeeditor_with_correct_format(qtbot):
     editor = CollectionsEditorTableView(None, {'df': df})
     qtbot.addWidget(editor)
     CONF.set('variable_explorer', 'dataframe_format', '10d')
-    editor.delegate.createEditor(None, None, editor.model.index(0, 3))
+    editor.delegate.createEditor(None, None, editor.model().index(0, 3))
     dataframe_editor = next(iter(editor.delegate._editors.values()))['editor']
     qtbot.addWidget(dataframe_editor)
     dataframe_editor.dataModel._format_spec == '10d'
 
 
 def test_collectionsmodel_with_two_ints():
-    coll = {'x': 1, 'y': 2}
+    coll = {'y': 2, 'x': 1}
     cm = CollectionsModel(MockParent(), coll)
 
     assert cm.rowCount() == 2
     assert cm.columnCount() == 4
-    # dict is unordered, so first row might be x or y
-    assert data(cm, 0, 0) in {'x',
-                              'y'}
-    if data(cm, 0, 0) == 'x':
-        row_with_x = 0
-        row_with_y = 1
-    else:
-        row_with_x = 1
-        row_with_y = 0
-    assert data(cm, row_with_x, 1) == 'int'
-    assert data(cm, row_with_x, 2) == 1
-    assert data(cm, row_with_x, 3) == '1'
-    assert data(cm, row_with_y, 0) == 'y'
-    assert data(cm, row_with_y, 1) == 'int'
-    assert data(cm, row_with_y, 2) == 1
-    assert data(cm, row_with_y, 3) == '2'
+
+    # dict is sorted by insertion order
+    assert data(cm, 0, 0) == 'y'
+    assert data(cm, 0, 1) == 'int'
+    assert data(cm, 0, 2) == 1
+    assert data(cm, 0, 3) == '2'
+    assert data(cm, 1, 0) == 'x'
+    assert data(cm, 1, 1) == 'int'
+    assert data(cm, 1, 2) == 1
+    assert data(cm, 1, 3) == '1'
 
 
 def test_collectionsmodel_with_index():
@@ -350,10 +349,21 @@ def test_shows_dataframeeditor_when_editing_index(monkeypatch):
         coll = {'rng': rng}
         editor = CollectionsEditorTableView(None, coll)
         editor.delegate.createEditor(None, None,
-                                     editor.model.index(0, 3))
+                                     editor.model().index(0, 3))
         mockDataFrameEditor_instance.show.assert_called_once_with()
 
 
+def test_shows_collectioneditor_when_editing_frozenset():
+    fs = frozenset('Spyder')
+    editor = CollectionsEditorTableView(None, {'fs': fs})
+    name_to_patch = 'spyder.widgets.collectionseditor.CollectionsEditor'
+    with patch(name_to_patch) as MockCollectionsEditor:
+        editor.delegate.createEditor(
+            None, None, editor.model().index(0, 3)
+        )
+    MockCollectionsEditor.return_value.show.assert_called_once_with()
+
+
 def test_sort_numpy_numeric_collectionsmodel():
     if parse(numpy.__version__) >= parse("2.0.0"):
         np20 = True
@@ -522,6 +532,85 @@ def test_sort_and_fetch_collectionsmodel_with_many_rows():
     assert cm.rowCount() == len(coll)
 
 
+def test_dict_in_tableview_sorting(qtbot):
+    """
+    Test clicking on a column header in an editor showing a dict cycles
+    through sorting in ascending, descending and insertion order.
+    """
+    my_dict = {2: 3, 3: 1, 1: 2}
+    editor = CollectionsEditorTableView(None, my_dict)
+    qtbot.addWidget(editor)
+    editor.show()
+
+    # Test that dict is displayed in insertion order
+    assert data_col(editor.model(), 0) == [2, 3, 1]
+    assert data_col(editor.model(), 3) == ['3', '1', '2']
+
+    # Click on header of first column
+    header = editor.horizontalHeader()
+    x_col0 = header.sectionPosition(0) + header.sectionSize(0) // 2
+    with qtbot.waitSignal(header.sectionClicked, timeout=200):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col0, 1)
+        )
+
+    # Test that dict is sorted by key
+    assert data_col(editor.model(), 0) == [1, 2, 3]
+    assert data_col(editor.model(), 3) == ['2', '3', '1']
+
+    # Click on header of first column
+    with qtbot.waitSignal(header.sectionClicked, timeout=200):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col0, 1)
+        )
+
+    # Test that dict is sorted by key in reverse order
+    assert data_col(editor.model(), 0) == [3, 2, 1]
+    assert data_col(editor.model(), 3) == ['1', '3', '2']
+
+    # Click on header of first column
+    with qtbot.waitSignal(header.sectionClicked, timeout=200):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col0, 1)
+        )
+
+    # Test that dict is displayed in insertion order
+    assert data_col(editor.model(), 0) == [2, 3, 1]
+    assert data_col(editor.model(), 3) == ['3', '1', '2']
+
+    # Click on header of fourth column
+    x_col3 = header.sectionPosition(3) + header.sectionSize(3) // 2
+    with qtbot.waitSignal(header.sectionClicked, timeout=2000):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col3, 1)
+        )
+
+    # Test that dict is sorted by value
+    assert data_col(editor.model(), 0) == [3, 1, 2]
+    assert data_col(editor.model(), 3) == ['1', '2', '3']
+
+    # Click on header of fourth column
+    with qtbot.waitSignal(header.sectionClicked, timeout=200):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col3, 1)
+        )
+
+    # Test that dict is sorted by value in reverse order
+    assert data_col(editor.model(), 0) == [2, 1, 3]
+    assert data_col(editor.model(), 3) == ['3', '2', '1']
+
+    # Click on header of first column
+    header = editor.horizontalHeader()
+    with qtbot.waitSignal(header.sectionClicked, timeout=200):
+        qtbot.mouseClick(
+            header.viewport(), Qt.LeftButton, pos=QPoint(x_col0, 1)
+        )
+
+    # Test that dict is sorted by key
+    assert data_col(editor.model(), 0) == [1, 2, 3]
+    assert data_col(editor.model(), 3) == ['2', '3', '1']
+
+
 def test_rename_and_duplicate_item_in_collection_editor():
     collections = {'list': ([1, 2, 3], False, True),
                    'tuple': ((1, 2, 3), False, False),
@@ -531,7 +620,7 @@ def test_rename_and_duplicate_item_in_collection_editor():
         editor = CollectionsEditorTableView(None, coll)
         assert editor.rename_action.isEnabled()
         assert editor.duplicate_action.isEnabled()
-        editor.setCurrentIndex(editor.model.index(0, 0))
+        editor.setCurrentIndex(editor.model().index(0, 0))
         editor.refresh_menu()
         assert editor.rename_action.isEnabled() == rename_enabled
         assert editor.duplicate_action.isEnabled() == duplicate_enabled
@@ -618,7 +707,7 @@ def test_collectioneditor_refresh_nested():
     editor = CollectionsEditor(None, data_function=lambda: new_list)
     editor.setup(old_list)
     view = editor.widget.editor
-    view.edit(view.model.index(3, 3))
+    view.edit(view.model().index(3, 3))
     nested_editor = list(view.delegate._editors.values())[0]['editor']
     assert nested_editor.get_value() == (4, 5)
     nested_editor.widget.refresh_action.trigger()
@@ -636,12 +725,12 @@ def test_edit_datetime(monkeypatch):
 
     # Test that the NaT value cannot be edited on the variable explorer
     editor_list_value = editor_list.delegate.createEditor(
-        None, None, editor_list.model.index(0, 3))
+        None, None, editor_list.model().index(0, 3))
     assert editor_list_value is None
 
     # Test that a date can be edited on the variable explorer
     editor_list_value = editor_list.delegate.createEditor(
-        None, None, editor_list.model.index(1, 3))
+        None, None, editor_list.model().index(1, 3))
     assert isinstance(editor_list_value, QDateEdit)
 
 
@@ -681,31 +770,31 @@ def test_edit_mutable_and_immutable_types(monkeypatch):
 
     # Directly editable values inside list
     editor_list_value = editor_list.delegate.createEditor(
-        None, None, editor_list.model.index(0, 3))
+        None, None, editor_list.model().index(0, 3))
     assert editor_list_value is not None
     assert MockQLineEdit.call_count == 1
 
     # Text Editor for long text inside list
     editor_list.delegate.createEditor(None, None,
-                                      editor_list.model.index(1, 3))
+                                      editor_list.model().index(1, 3))
     assert MockTextEditor.call_count == 2
     assert not MockTextEditor.call_args[1]["readonly"]
 
     # Datetime inside list
     editor_list_datetime = editor_list.delegate.createEditor(
-        None, None, editor_list.model.index(2, 3))
+        None, None, editor_list.model().index(2, 3))
     assert editor_list_datetime is not None
     assert MockQDateTimeEdit.call_count == 1
 
     # List inside list
     editor_list.delegate.createEditor(None, None,
-                                      editor_list.model.index(3, 3))
+                                      editor_list.model().index(3, 3))
     assert mockCollectionsEditor_instance.show.call_count == 1
     assert not mockCollectionsEditor_instance.setup.call_args[1]["readonly"]
 
     # Tuple inside list
     editor_list.delegate.createEditor(None, None,
-                                      editor_list.model.index(4, 3))
+                                      editor_list.model().index(4, 3))
     assert mockCollectionsEditor_instance.show.call_count == 2
     assert mockCollectionsEditor_instance.setup.call_args[1]["readonly"]
 
@@ -714,31 +803,31 @@ def test_edit_mutable_and_immutable_types(monkeypatch):
 
     # Directly editable values inside tuple
     editor_tup_value = editor_tup.delegate.createEditor(
-        None, None, editor_tup.model.index(0, 3))
+        None, None, editor_tup.model().index(0, 3))
     assert editor_tup_value is None
     assert MockQLineEdit.call_count == 1
 
     # Text Editor for long text inside tuple
     editor_tup.delegate.createEditor(None, None,
-                                     editor_tup.model.index(1, 3))
+                                     editor_tup.model().index(1, 3))
     assert MockTextEditor.call_count == 4
     assert MockTextEditor.call_args[1]["readonly"]
 
     # Datetime inside tuple
     editor_tup_datetime = editor_tup.delegate.createEditor(
-        None, None, editor_tup.model.index(2, 3))
+        None, None, editor_tup.model().index(2, 3))
     assert editor_tup_datetime is None
     assert MockQDateTimeEdit.call_count == 1
 
     # List inside tuple
     editor_tup.delegate.createEditor(None, None,
-                                     editor_tup.model.index(3, 3))
+                                     editor_tup.model().index(3, 3))
     assert mockCollectionsEditor_instance.show.call_count == 3
     assert mockCollectionsEditor_instance.setup.call_args[1]["readonly"]
 
     # Tuple inside tuple
     editor_tup.delegate.createEditor(None, None,
-                                     editor_tup.model.index(4, 3))
+                                     editor_tup.model().index(4, 3))
     assert mockCollectionsEditor_instance.show.call_count == 4
     assert mockCollectionsEditor_instance.setup.call_args[1]["readonly"]
 
@@ -755,7 +844,7 @@ def test_edit_large_int(monkeypatch, exponent, error_expected):
     """
     num = 10 ** exponent + 1
     editor = CollectionsEditorTableView(None, [num])
-    index = editor.source_model.index(0, 3)
+    index = editor.model().index(0, 3)
 
     with patch(
         'spyder.plugins.variableexplorer.widgets'
@@ -861,7 +950,7 @@ def test_editor_parent_set(monkeypatch):
                                       MockObjectExplorer,
                                       MockTextEditor]):
         col_editor.delegate.createEditor(col_editor.parent(), None,
-                                         col_editor.model.index(idx, 3))
+                                         col_editor.model().index(idx, 3))
         assert mock_class.call_count == 1 + (idx // 4)
         assert mock_class.call_args[1]["parent"] is test_parent
 
@@ -1016,7 +1105,7 @@ def test_collectionseditor_when_clicking_on_header_and_large_rows(qtbot):
         qtbot.mouseClick(header.viewport(), Qt.LeftButton, pos=QPoint(1, 1))
 
     # Assert data was sorted correctly.
-    assert data(view.model, 0, 0) == 9999
+    assert data(view.model(), 0, 0) == 9999
 
     editor.accept()
 
@@ -1071,24 +1160,30 @@ def test_dicts_natural_sorting_mixed_types():
     editor = CollectionsEditor()
     editor.setup(dictionary)
     cm = editor.widget.editor.source_model
-    cm.sort(0)
     keys = cm.keys
     types = cm.types
     sizes = cm.sizes
 
-    assert keys == ['aStr', 'DSeries', 'kDict']
-    assert types == ['str', 'Series', 'dict']
-    assert sizes == [str_size, (0,), 2]
+    # Initially sorted by insertion order
+    assert keys == ['DSeries', 'aStr', 'kDict']
+    assert types == ['Series', 'str', 'dict']
+    assert sizes == [(0,), str_size, 2]
 
-    assert data_table(cm, 3, 3) == [['aStr', 'DSeries', 'kDict'],
-                                    ['str', 'Series', 'dict'],
-                                    [str_size, '(0,)', 2]]
+    assert data_table(cm, 3, 3) == [
+        ["DSeries", "aStr", "kDict"],
+        ["Series", "str", "dict"],
+        ["(0,)", str_size, 2],
+    ]
 
     # insert an item and check that it is still sorted correctly
     editor.widget.editor.new_value('List', [1, 2, 3])
-    assert data_table(cm, 4, 3) == [['aStr', 'DSeries', 'kDict', 'List'],
-                                    ['str', 'Series', 'dict', 'list'],
-                                    [str_size, '(0,)', 2, 3]]
+    assert data_table(cm, 4, 3) == [
+        ["DSeries", "aStr", "kDict", "List"],
+        ["Series", "str", "dict", "list"],
+        ["(0,)", str_size, 2, 3],
+    ]
+
+    # now sort by key
     cm.sort(0)
     assert data_table(cm, 4, 3) == [['aStr', 'DSeries', 'kDict', 'List'],
                                     ['str', 'Series', 'dict', 'list'],
@@ -1110,7 +1205,8 @@ def test_dicts_natural_sorting_mixed_types():
 def test_collectioneditor_plot(qtbot):
     """
     Test that plotting a list from the collection editor calls the .plot()
-    function in the associated namespace browser.
+    function in the associated namespace browser and that the executing
+    `plot_function` plots the list.
     """
     my_list = [4, 2]
     mock_namespacebrowser = Mock()
@@ -1119,7 +1215,13 @@ def test_collectioneditor_plot(qtbot):
     qtbot.addWidget(cew)
 
     cew.editor.plot('list', 'plot')
-    mock_namespacebrowser.plot.assert_called_once_with(my_list, 'plot')
+    mock_namespacebrowser.plot.assert_called_once()
+
+    plot_function = mock_namespacebrowser.plot.call_args.args[0]
+    mock_figure = Mock()
+    plot_function(mock_figure)
+
+    mock_figure.subplots.return_value.plot.assert_called_once_with(my_list)
 
 
 def test_collectionseditor_select_row_button(qtbot):
